c	*************************************************************
c	Subroutine to read molecular line data from a file
c	Adapted version from the Leiden workshop
c	*************************************************************
	subroutine readspecies(filename,nlin,lev,kerr)
	implicit none
	integer nlin,lev, kerr,ioerror
	character*80 filename

c	*************************************************************
c	Original arrays for reading
c	They have to large enough to read all data provided.
c	In general only a subset of them will be used
c	*************************************************************
	integer NLEVMAX,NLINMAX,NCOLMAX,NTEMPMAX
c	Maximum number of molecular levels
	parameter(NLEVMAX=50)
c	Maximum number of molecular line transitions
	parameter(NLINMAX=200)
c	Maximum number of molecular collisional transitions
	parameter(NCOLMAX=1000)
c	Maximum number of temperatures for which Kud given
	parameter(NTEMPMAX=10)
c
	real energy_lev(NLEVMAX), gdeg(NLEVMAX)
	real Aud(NLINMAX), Kud(NCOLMAX,NTEMPMAX), Temp_Kud(NTEMPMAX)
	integer nlines,ncoltrans,nlevels,ncttemp
	integer lnum(NLEVMAX),lev_up(NLINMAX),lev_down(NLINMAX),
     %        lev_ct_up(NCOLMAX),lev_ct_down(NCOLMAX)
	character*8 names(NLEVMAX)

c	Auxiliary variables for reading
	integer i,itemp,lmax,lmin
	character*80 dummyline,leer

c	***********************************************************
c	New arrays as they are used in the code
c	***********************************************************
	include 'fsizes.inc'
	real molmass,energy(nlev),stgew(nlev)
	real fr(ntra),eina(ntra),einb(ntra),gewratio(ntra)
	real temp(ntempmax),collrates((nlev*(nlev-1))/2,ntempmax)
	real clight,cst1,cst2
	integer lfrom(ntra),lto(ntra)
	integer icount,ntemp
	logical downwards
	character*8 levnam(nlev)

	common /molecule/ molmass, energy, stgew, levnam
	common /coeffein/ fr,eina,einb,gewratio
	common /crates/ downwards,ntemp,temp,collrates
	common /lintable/ lfrom, lto
	save /molecule/, /coeffein/, /crates/, /lintable/

c	***********************************************************
c	Translation coefficients into my units
c	***********************************************************
c	clight = light velocity in cm/s
c	cst1 = clight*h/kB  (Translate energies into K)
c	cst2=(8*pi/c^2)^(1/3) (Tranlation from Einstein As to Bs)
c	***********************************************************
	data clight,cst1,cst2 / 2.997925e10, 1.4387687, 3.03529e-7/

c	Error messages
13	format(' Name of the molecular data file: ',$)
12	format(/,' Reading molecular data file: ',A40)
14	format(/,' Number of levels in the molecule file too large !')
15	format(/,' Number of lines in the molecule file too large !')
16	format(/,' Number of temperatures in the molecule file too',
     %          ' large !')
17	format(/,' Number of collision rates in the molecule file',
     %          ' too large !')
18	format(/,' Inconsistency in the level numbering detected !')
19	format(/,' Wrong order of level numbering detected !')
c	Input formats
41	format(A1)
42	format(A80)

c	***********************************************************
c	Start reading
c	***********************************************************
	kerr=0
	leer=' '
100	if (filename.eq.leer) then
	write(6,13)
	read (5,42) filename
	else
	write(6,12) filename
	endif
	open(unit=14,err=120,file=filename,status='old')
c	Global parameters
	read (14,42,err=110,end=110) dummyline
	read (14,42,err=110,end=110) dummyline
	read (14,*,err=110,end=110) molmass
	read (14,42,err=110,end=110) dummyline
	read (14,*,err=110,end=110) nlevels
	read (14,42,err=110,end=110) dummyline
	read (14,*,err=110,end=110) nlines
c	Check for readability
	if (nlevels.gt.nlevmax) then
	 write(6,14)
	 goto 500
	endif
	if (nlines.gt.nlinmax) then
	 write(6,15)
	 goto 500
	endif
c	Read all level parameters
	read (14,42,err=110,end=110) dummyline
	do i=1,nlevels
	read (14,*,err=110,end=110) 
     %    lnum(i),gdeg(lnum(i)),energy_lev(lnum(i)),names(lnum(i))
	enddo
c	***********************************************************
c	Consistency check for the level numbers
c	It is assumed that levels are numbered starting 1 and
c	sequentially up to nlevels
c	***********************************************************
	lmax=0
	lmin=nlevmax
	do i=1,nlevels
	lmax=max(lmax,lnum(i))
	lmin=min(lmin,lnum(i))
	enddo
	if ((lmin.ne.1).or.(lmax.ne.nlevels)) then
	 write(6,18)
	 goto 500
	endif
c	***********************************************************
c	Now check for the correct order of level numbering
c	***********************************************************
	do i=2,nlevels
	if (energy_lev(i).lt.energy_lev(i-1)) then
	  write(6,19)
	  goto 500
	endif
	enddo

c	***********************************************************
c	First reduction to used values
c	***********************************************************
	lev=min(nlevels,nlev)
	do i=1,lev
	  energy(i)=energy_lev(i)*cst1
	  stgew(i)=gdeg(i)
	  levnam(i)=names(i)
	enddo

c	***********************************************************
c	Further input - Einstein A's
c	***********************************************************
	read (14,42,err=110,end=110) dummyline
	do i=1,nlines
	read(14,*,err=110,end=110) 
     %    lev_up(i),lev_down(i),Aud(i)
	enddo
c	***********************************************************
c	Consistency check for the lines
c	***********************************************************
	lmax=-nlinmax
	lmin=nlinmax
	do i=1,nlines
	lmin=min(lmin,lev_down(i))
	lmax=max(lmax,lev_down(i)-lev_up(i))
	enddo
	if ((lmin.lt.1).or.(lmax.gt.(-1))) then
	 write(6,19)
	 goto 500
	endif
c	**************************************************************
c	Second reduction step for Einstein coefficients
c	**************************************************************
	i=0
	nlin=0
200	continue
	  i=i+1
	  if (lev_up(i).le.lev) then
	  nlin=nlin+1
	  lfrom(nlin)=lev_up(i)
	  lto(nlin)=lev_down(i)
	  fr(nlin)=(energy_lev(lev_up(i))-energy_lev(lev_down(i)))
     %               *clight
	  gewratio(nlin)=gdeg(lev_up(i))/gdeg(lev_down(i))
	  eina(nlin)=Aud(i)
	  einb(nlin)=eina(nlin)/(cst2*fr(nlin))**3
	  endif
	if ((nlin.lt.ntra).and.(i.lt.nlines)) goto 200

c	**************************************************************
c	Now we read the collisional rate coefficients
c	**************************************************************
	read (14,42,err=110,end=110) dummyline
	read (14,42,err=110,end=110) dummyline
	read (14,*,err=110,end=110) ncttemp
c	Check for readability
	if (ncttemp.gt.ntempmax) then
	 write(6,16)
	 goto 500
	endif

c	Read temperatures
	read (14,42,err=110,end=110) dummyline
	do i=1,ncttemp
	 read (14,*,err=110,end=110) temp_kud(i)
	enddo
	read (14,42,err=110,end=110) dummyline
	read (14,*,err=110,end=110) ncoltrans
	if (ncoltrans.gt.ncolmax) then
	 write(6,17)
	 goto 500
	endif
c	Main loop for reading collision parameters
	read (14,42,err=110,end=110) dummyline
	do i=1,ncoltrans
	 read (14,*,err=110,end=110) lev_ct_up(i),lev_ct_down(i),
     %        (Kud(i,itemp),itemp=1,ncttemp)
	enddo
	close(14)

c	**************************************************************
c	Final reduction
c	**************************************************************
	downwards=(lev_ct_up(1).gt.lev_ct_down(1))
	ntemp=ncttemp
	do itemp=1,ntemp
	   temp(itemp)=Temp_Kud(itemp)
	enddo
	do i=1,lev*(lev-1)/2
	do itemp=1,ntemp
	   collrates(i,itemp)=0.0
	enddo
	enddo
	if (downwards) then
	 do i=1,ncoltrans
	   if ((lev_ct_up(i).le.lev).and.
     %                      (lev_ct_up(i).gt.lev_ct_down(i))) then
	   icount=((lev_ct_up(i)-1)*(lev_ct_up(i)-2))/2+lev_ct_down(i)
	   do itemp=1,ntemp
	     collrates(icount,itemp)=Kud(i,itemp)
	   enddo
	   endif
	 enddo
	else
	 do i=1,ncoltrans
	   if ((lev_ct_down(i).le.lev).and.
     %                      (lev_ct_down(i).gt.lev_ct_up(i))) then
	   icount=((lev_ct_down(i)-1)*(lev_ct_down(i)-2))/2+lev_ct_up(i)
	   do itemp=1,ntemp
	     collrates(icount,itemp)=Kud(i,itemp)
	   enddo
	   endif
	 enddo
	endif
	return

c	*************************************************************
c	Error handling
c	*************************************************************	
500	filename=leer
	close(14)
	goto 100
120	filename=leer
	goto (100) ioerror(filename,1)
	kerr=1
	return
110	filename=leer
	goto (100) ioerror(filename,2)
	kerr=1
	return

	end

c	***********************************************************
c	Calling routine for the computation of the collisional
c	rate coefficients
c	Calls either the CS or the CO routine
c	The calling program does not know about special molecules
c	***********************************************************
	real function cij(T,J,K)
	implicit none
	real T
	integer j,k,ntempmax

	include 'fsizes.inc'
	parameter (ntempmax=10)
	real molmass, energy(nlev),stgew(nlev),coldown
	real temp(ntempmax), coll((nlev*(nlev-1))/2,ntempmax)
	logical downwards
	integer jt,ilocat,jup,jlow,icount,ntemp
	character*8 levnam(nlev)

        common /molecule/ molmass, energy, stgew, levnam
        common /crates/ downwards,ntemp,temp,coll
        save /molecule/, /crates/

c	**************************************************************
c	Throw away nontransitional collisions
c	**************************************************************
	if (j.eq.k)  then
	cij = 0.
	return
	endif
c	**************************************************************
c	Level numbers
c	**************************************************************
	jup   = max(j,k)
	jlow  = min(j,k)
	icount=((jup-1)*(jup-2))/2+jlow
c	**************************************************************
c	Square root extrapolation below the lowest rate point
c	************************************************************** 
	if (t.lt.temp(1)) then
	coldown = coll(icount,1)*sqrt(t/temp(1))
c	**************************************************************
c	Linear interpolation for upward transition from the table
c	assumption of detailed balance for downward transitions
c	cul=clu*gl/gu*exp(h*ny/kT)
c	**************************************************************
	else
	jt = ilocat(temp, ntemp, T)
	coldown = coll(icount,jt) +(t-temp(jt))/(temp(jt+1)-temp(jt))
     %               *(coll(icount,jt+1)-coll(icount,jt))
	endif
	if (coldown.gt.0.0) then
c	transitions in the opposite direction
	if (downwards.eqv.(j.eq.jlow)) then
	  cij = coldown*stgew(k)/stgew(j)*exp((energy(j)-energy(k))/T)
	else
	  cij = coldown
	endif
	else
	  cij = 0.0
	endif
c	**************************************************************
c	To get cm^3/s all values have to be multiplied by 1e-10
c	**************************************************************
	return
	end

c	*************************************************************
c	function to select the next lower index in a field of
c	parameters for a given parameter
c	x < xy(1) < xy(n) -> j = 1
c	xy(j) < x < xy(j+1) -> j = j
c	x > xy(n) ->  j = n-1
c	It has to be guaranteed that xy is ordered positively
c	x will be increased by a small factor to find close matches
c	*************************************************************
	integer function ilocat(xy, n, x)
	implicit none
	real xy(*),x,xh
	integer n,jlow,jup,jm

	jlow = 1
	jup  = n
	xh = x*1.000002

100	jm = (jup + jlow) / 2
	if (xh.gt.xy(jm)) then
	 jlow = jm
	else
	 jup  = jm
	end if
	if (jup-jlow.gt.1) goto 100
       
	if (jlow.eq.n) then
	ilocat = jlow - 1
	else
	ilocat = jlow
	endif
	return
	end

