c	***********************************************************
c	Subroutine providing the interface from a variable program
c	to universal fits writing routines
c	***********************************************************
	subroutine writecub(ibeam,nx,ny,nff,dx,dy,df,f0,info,lfile,kerr)
	implicit none

	integer nx,ny,nff,kerr

	include 'fsizes.inc'
	real ibeam(ns,ns,2*nf+1),data(ns*ns*(2*nf+1))
	real dx,dy,df,f0,convert
	character*80 lfile
	integer ndim,nax(3)
	real v0(0:3),dv(0:3)
	character*8 units(0:3)
	character*40 comm(0:3)
	character*140 info
	integer i,k,l

c	Translate from arcsec to degree
	data convert / 0.00027777/

	ndim=3
	nax(1)=nx
	nax(2)=ny
	nax(3)=nff

	units(0)='K'
	comm(0)='Brightness temperature'
	units(1)='RA---TAN'
	comm(1)='in degrees'
	units(2)='DEC--TAN'
	comm(2)='in degrees'
	units(3)='VELOCITY'
	comm(3)='in km/s'

	v0(0)=0.0
	dv(0)=1.0
	v0(1)=0.0
	dv(1)=dx*convert
	v0(2)=0.0
	dv(2)=dy*convert
	v0(3)=f0
	dv(3)=df

	do 100 l=1,nff
	do 100 k=1,ny
	do 100 i=1,nx
	data(((l-1)*ny+(k-1))*nx+i)=ibeam(i,k,l)
100	continue

	kerr=0
	call savefits(ndim,nax,data,v0,dv,units,comm,info,lfile,kerr)
	return
	end

c	***********************************************************
c	Subroutine providing the interface from a variable program
c	to universal fits writing routines
c	***********************************************************
	subroutine cubread(ibeam,nx,ny,nff,dx,dy,df,f0,lfile,kerr)
	implicit none

	integer nx,ny,nff,kerr

	include 'fsizes.inc'
	real ibeam(ns,ns,2*nf+1),data(ns*ns*(2*nf+1))
	real dx,dy,df,f0,convert
	character*80 lfile
	integer ndim,nax(3)
	real v0(0:3),dv(0:3)
	character*8 units(0:3)
	character*40 comm(0:3)
	integer i,k,l

c	Translate from arcsec to degree
	data convert / 0.00027777/

15	format(' Grid read: ',I3,'x',I3,'x',I3)

	kerr=0
	ndim=3
	call readfits(ndim,nax,data,v0,dv,units,comm,lfile,kerr)
	if (kerr.ne.0) return

	nx=nax(1)
	ny=nax(2)
	nff=nax(3)
	write(6,15) nx,ny,nff
	dx=dv(1)/convert
	dy=dv(2)/convert
	df=dv(3)
	f0=v0(3)

	do 100 l=1,nff
	do 100 k=1,ny
	do 100 i=1,nx
	ibeam(i,k,l)=data(((l-1)*ny+(k-1))*nx+i)
100	continue
c	Correct for missing scales occuring in some Heidelberg data
	if (dx.eq.0.0) dx=1.0
	if (dy.eq.0.0) dy=1.0
	if (df.eq.0.0) df=1.0

	return
	end

c	***********************************************************
c	Subroutine providing the interface from a variable program
c	to universal fits writing routines
c	***********************************************************
	subroutine mapread(ibeam,nx,ny,x0,y0,dx,dy,lfile,kerr)
	implicit none

	integer nx,ny,kerr

	include 'fsizes.inc'
	real ibeam(ns,2*nf+1),data(ns*ns)
	real x0,y0,dx,dy,convert
	character*80 lfile
	integer ndim,nax(3)
	real v0(0:3),dv(0:3)
	character*8 units(0:3)
	character*40 comm(0:3)
	integer i,k

c	Translate from arcsec to degree
	data convert / 0.00027777/

15	format(' Grid read: ',I3,'x',I3)

	kerr=0
	ndim=2
	call readfits(ndim,nax,data,v0,dv,units,comm,lfile,kerr)
	if (kerr.ne.0) return

	nx=nax(1)
	ny=nax(2)
	write(6,15) nx,ny

	v0(0)=0.0
	dv(0)=1.0
	x0=v0(1)/convert
	dx=dv(1)/convert
	y0=v0(2)/convert
	dy=dv(2)/convert

	do 100 k=1,ny
	do 100 i=1,nx
	ibeam(i,k)=data((k-1)*nx+i)
100	continue
	return
	end

c	***********************************************************
c	Subroutine providing the interface from a variable program
c	to universal fits writing routines
c	***********************************************************
	subroutine writemap(ibeam,nx,ny,x0,y0,dx,dy,info,lfile,kerr)
	implicit none

	integer nx,ny,kerr

	include 'fsizes.inc'
	real ibeam(ns,2*nf+1),data(ns*ns)
	real x0,y0,dx,dy,convert
	character*80 lfile
	integer ndim,nax(3)
	real v0(0:3),dv(0:3)
	character*8 units(0:3)
	character*40 comm(0:3)
	character*140 info
	integer i,k

c	Translate from arcsec to degree
	data convert / 0.00027777/

	ndim=2
	nax(1)=nx
	nax(2)=ny

	units(0)='K'
	comm(0)='Brightness temperature'
	units(1)='RA---TAN'
	comm(1)='in degrees'
	units(2)='DEC--TAN'
	comm(2)='in degrees'

	v0(0)=0.0
	dv(0)=1.0
	v0(1)=x0*convert
	dv(1)=dx*convert
	v0(2)=y0*convert
	dv(2)=dy*convert

	do 100 k=1,ny
	do 100 i=1,nx
	data((k-1)*nx+i)=ibeam(i,k)
100	continue

	kerr=0
	call savefits(ndim,nax,data,v0,dv,units,comm,info,lfile,kerr)
	return
	end

c	***********************************************************
c	Write a position-velocity map
c	***********************************************************
	subroutine writevel(ibeam,nx,nff,x0,f0,dx,df,info,lfile,kerr)
	implicit none

	integer nx,nff,kerr

	include 'fsizes.inc'
	real ibeam(ns,2*nf+1),data(ns*(2*nf+1))
	real x0,f0,dx,df,convert
	character*80 lfile
	integer ndim,nax(3)
	real v0(0:3),dv(0:3)
	character*8 units(0:3)
	character*40 comm(0:3)
	character*140 info
	integer i,k

c	Translate from arcsec to degree
	data convert / 0.00027777/

	ndim=2
	nax(1)=nx
	nax(2)=nff

	units(0)='K'
	comm(0)='Brightness temperature'
	units(1)='RA---TAN'
	comm(1)='in degrees'
	units(2)='FREQ'
	comm(2)=' in km/s'

	v0(0)=0.0
	dv(0)=1.0
	v0(1)=x0*convert
	dv(1)=dx*convert
	v0(2)=f0
	dv(2)=df

	do 100 k=1,nff
	do 100 i=1,nx
	data((k-1)*nx+i)=ibeam(i,k)
100	continue

	kerr=0
	call savefits(ndim,nax,data,v0,dv,units,comm,info,lfile,kerr)
	return
	end

c	*************************************************************
c	Subroutine to write an arbitrary one-field  fits file
c	***********************************************************
	subroutine savefits(ndim,nx,daten,v0,dv,units,comments,
     %					         info,lfile,kerr)
	implicit none
	integer ndim,nx(3),kerr
	real v0(0:3),dv(0:3)
	character*8 units(0:3)
	character*40 comments(0:3)
	real daten(*)
	character*80 lfile,leer
	character*140 info

	integer i,ioerror
	integer status,blocksize,bitpix,naxis,naxes(3)
	integer group,fpixel,nelements
	logical simple,extend
	character*6 ctype(3),crpix(3),crval(3),cdelt(3)

	data ctype /'CTYPE1','CTYPE2','CTYPE3'/
	data crpix /'CRPIX1','CRPIX2','CRPIX3'/
	data crval /'CRVAL1','CRVAL2','CRVAL3'/
	data cdelt /'CDELT1','CDELT2','CDELT3'/

c	Input formats
13      format(/,' Name of the output FITS file: ',$)
42	format(A80)

c	*************************************************************
c	Open the file for writing
c	*************************************************************
	blocksize=1
	leer=' '
	if (lfile.ne.leer) goto 200
350	write(6,13)
	read (5,42) lfile
200	status=0
	call ftinit(14,lfile,blocksize,status)
	if (status.ne.0) goto (150,200,350) ioerror(lfile,3)+2
c	*************************************************************
c	Required header
c	*************************************************************
	simple=.true.
	bitpix=-32
	extend=.true.
	naxis=ndim
	do 210 i=1,ndim
210	 naxes(i)=nx(i)
	call ftphpr(14,simple,bitpix,naxis,naxes,0,1,extend,status)
c	*************************************************************
c	Extended header
c	*************************************************************
	call ftpkys(14,'BUNIT ',units(0),comments(0),status)
	call ftpkye(14,'BZERO ',v0(0),4,' ',status)
	call ftpkye(14,'BSCALE',dv(0),4,' ',status)
	do 220 i=1,ndim
	 call ftpkys(14,ctype(i),units(i),comments(i),status)
	 call ftpkyj(14,crpix(i),1,' ',status)
	 call ftpkye(14,crval(i),v0(i),4,' ',status)
	 call ftpkye(14,cdelt(i),dv(i),4,' ',status)
220	continue
	call ftpcom(14,info,status)
	call ftpdat(14,status)
c	************************************************************
c	Write field
c	************************************************************
	group=1
	fpixel=1
	nelements=1
	do 110 i=1,ndim
110	 nelements=nelements*nx(i)
	call ftppre(14,group,fpixel,nelements,daten,status)
	call ftclos(14,status)
	if (status.gt.0) then
	 call printerror(status)
	 kerr=1
	endif
	return

c	*************************************************************
c	Error handling
c	*************************************************************
150	kerr=1
	return
	end

c	***********************************************************
c	Subroutine to write out the line profiles in a form that
c	can be read by greg or gnuplot
c	***********************************************************
	subroutine readfits(ndim,nx,data,v0,dv,units,comments,
     %							lfile,kerr)
	implicit none
	integer ndim,nx(3),kerr
	real v0(0:3),dv(0:3)
	character*8 units(0:3)
	character*40 comments(0:3)
	real data(*)
	character*80 lfile, leer

	integer i,ioerror
	integer status,blocksize,readwrite,bitpix,naxis,naxes(3)
	integer group,fpixel,nelements,pc,gc
	logical simple,extend,anyf
	real null
	character*6 ctype(3),crval(3),cdelt(3)
	character*80 dummy

	data ctype /'CTYPE1','CTYPE2','CTYPE3'/
	data crval /'CRVAL1','CRVAL2','CRVAL3'/
	data cdelt /'CDELT1','CDELT2','CDELT3'/

c	Input formats
13      format(/,' Name of the input FITS file: ',$)
42	format(A80)

c	*************************************************************
c	Open the file
c	*************************************************************
	readwrite=0
	leer=' '
	if (lfile.ne.leer) goto 200
350	write(6,13)
	read (5,42) lfile
200	status=0
	call ftopen(14,lfile,readwrite,blocksize,status)
	if (status.ne.0) goto (150,200,350) ioerror(lfile,1)+2
c	*************************************************************
c	Required header
c	*************************************************************
	simple=.true.
	call ftghpr(14,3,simple,bitpix,naxis,naxes,pc,gc,extend,status)
	if ((status.ne.0).or.(naxis.ne.ndim))
     %			 goto (150,200,350) ioerror(lfile,2)+2
	do 210 i=1,ndim
210	 nx(i)=naxes(i)
c	*************************************************************
c	Extended header
c	*************************************************************
	call ftgkys(14,'BUNIT ',units(0),comments(0),status)
	call ftgkye(14,'BZERO ',v0(0),dummy,status)
	call ftgkye(14,'BSCALE',dv(0),dummy,status)
c	Heidelberg cubes often have no data scaling
c	This has to be caught as exception, not as error.
	if (status.ne.0) then
	units(0)='        '
	comments(0)='                                       '
	v0(0)=0.0
	dv(0)=1.0
	status=0
	endif
c	Other header parts
	do 220 i=1,ndim
	 call ftgkys(14,ctype(i),units(i),comments(i),status)
	 call ftgkye(14,crval(i),v0(i),dummy,status)
	 call ftgkye(14,cdelt(i),dv(i),dummy,status)
c       Exception handling
         if (status.ne.0) then
	  units(i)='        '
	  comments(i)='                                       '
	  v0(i)=0.0
	  dv(i)=1.0
	  status=0
         endif
220	continue
	write(6,*) ' Header read, status:', status
c	************************************************************
c	Read field
c	************************************************************
	group=1
	fpixel=1
	null=0.0
	nelements=1
	anyf=.false.
	do 110 i=1,ndim
110	 nelements=nelements*nx(i)
	call ftgpve(14,group,fpixel,nelements,null,data,anyf,status)
	if ((status.ne.0).or.(anyf)) 
     %			goto (150,200,350) ioerror(lfile,2)+2
	call ftclos(14,status)
	return

c	*************************************************************
c	Error handling
c	*************************************************************
150	kerr=1
	return
	end

c	*************************************************************
c	Subroutine for general file I/O error handling
c	*************************************************************
	integer function ioerror(lfile,inum)
	implicit none
	character*80 lfile
	character yn
	integer inum,kerr

41	format(A1)
51	format(/,' I cannot open the input file ',A80)
52	format(//,' Format error in input file ',A80)
53	format(/,' This file exists already: ',A80)
54	format(' Shall I replace it ? [y/n]: ',$)
56	format(' I cannot open the output file ',A80)
57	format(' Use another file ? [y/n]: ',$)
58	format(/,' File access denied! ')
55	format(' Continue ? [y/n]: ',$)

	ioerror=-1
	if (inum.eq.1) then
c	Error opening the input file
	write(6,51) lfile
	write(6,55)
	read (5,41) yn
	if ((yn.eq.'y').or.(yn.eq.'Y')) ioerror=1
	return
	else if (inum.eq.2) then
c	The file has not the structure required
	close(14)
	write(6,52) lfile
	write(6,55)
	read (5,41) yn
	if ((yn.eq.'y').or.(yn.eq.'Y')) ioerror=1
	return
	else if (inum.eq.3) then
c	Error opening the output file - Test if the file exists
	open(14,err=140,file=lfile,status='old')
	close(14)
	 write(6,53) lfile
	 write(6,54)
	 read (5,41) yn
	 if ((yn.ne.'y').and.(yn.ne.'Y')) goto 150
	kerr=0
	call deletefile(lfile,kerr)
	if (kerr.ne.0) then
	  write(6,58)
	  goto 150
	endif
	ioerror=0
	return
	endif

c	The file is not writable
140	write(6,56) lfile
150	write(6,57)
	read (5,41) yn
	if ((yn.eq.'y').or.(yn.eq.'Y')) ioerror=1
	return
	end

c	*********************************************************
c	A simple little routine to delete an unwanted file
c	*********************************************************
	subroutine deletefile(filename,status)
	implicit none
	integer status
	character*(*) filename

C	simply return if status is greater than zero
	if (status .gt. 0)return

c	Try opening the output file - Test if the file exists
	open(14,err=140,file=filename,status='old')
c	Erase file
	close(14,err=150,status='delete')
	status=0
	return

c	File does not exist
140	status=1
	return
c	Cannot erase file
150	status=100
	return
	end

c	**************************************************************
c	Routine from the FITSIO-Libraray 
c	**************************************************************
      subroutine printerror(status)

C     Print out the FITSIO error messages to the user

      integer status
      character errtext*30,errmessage*80

C     check if status is OK (no error); if so, simply return
      if (status .le. 0)return

C     get the text string which describes the error
 1    call ftgerr(status,errtext)
      print *,'FITSIO Error Status =',status,': ',errtext

C     read and print out all the error messages on the FITSIO stack
 2    call ftgmsg(errmessage)
      do while (errmessage .ne. ' ')
          print *,errmessage
          call ftgmsg(errmessage)
      end do
      end
