c	***********************************************************
c	Program to compute the line profiles from an fractal
c	density and velocity field
c	***********************************************************
	program fract_line
	implicit none
	integer nff,nx,ny,nz,lev,ilin,lin,lup,llo

	include 'fsizes.inc'
	real vz(ns,ns,ns),vt(ns,ns,ns),dco(ns,ns,ns)
	real vx(ns,ns,ns),vy(ns,ns,ns)
	real cptb(ntab,ntra),sctb(ntab,ntra)
	real cptabl(ntab),sctabl(ntab)
	real deltar,ubg1,dny,fwhm,dist,sigbeam,off,ztot
	real iout(ns,ns,-nf:nf),uout(-nf:nf,ntra)
	real ubg(ntra),uexi(ntab,ntra)
	real dh,t,tbeam,tback,factor,cfreq,dnyred,dvv
	real denstab(ndens),gradmom(ntab,6),regtab(ntab,6)
	real freqmom(ntab,2)
	real tau(ns,ns),values(4),vel
	integer inx(ns,ns,ns),point(3),lintab(ntra)
	integer kerr,ntt,l,i,inum,iter,nffred,theline,olev
	character*80 mol
	character*20 msg
	character yn

c	Translation from I to T: tbeam = h*c^2/(8*pi*k) [K*cm^2*s]
	data tbeam / 1.716215e9/
c	Cosmic background temperature
	data tback /2.7280/

c	Make storage for large fields static
	common /largefields/ dco,vx,vy,vz,vt
	save /largefields/

c	************************************************************
c	Formats for the error handling
c	************************************************************
12	format(/,' Number of frequency points insufficient to treat',
     %  ' the problem with the',/,' required accuracy !')
13	format(/,' No rate coefficients available for this',
     %  ' transition !')
26	format(/,' Beam size too large for the field !')
c       ************************************************************
c       Control output formats
c       ************************************************************
14	format(/,' Level populations computed - radiative transfer',
     %  ' is running ...')
19	format(/,' Excitation problem solved - ray tracing',
     %  ' is running ...')
15	format(/,' Radiative transfer computed - beam integration',
     %  ' is running ...')
24	format(/,' Radiative transfer computed - get level',
     %  ' populations ...')
23	format(/,' ',I2,'. iteration, ',I1,'. lambda step')
25      format(' Molecule can be treated with ',I3,' levels and ',
     %          I3, ' lines.')
16	format(/,' Frequency points in iteration: ',I4,
     %    ' and in RT: ',I4)
18	format(/,' Number of beams throughout the map: ',I3,'x',I3)
27	format(/,' Superfluid high levels removed: ',I3)
31	format(/,' Average, standard deviation, minimum, and maximum:')
32	format(' of the density field : ', 1P,4G11.3)
33	format(' of the velocity field: ', 1P,4G11.3)
34	format(' of the intensity field: ', 1P,4G11.3)
35	format(' of the optical depths:  ', 1P,4G11.3)
17	format(/,' Still another computation ?: ',$)
41	format(/,' Still another line ?: ',$)
42	format(/,' Upper level number: ',$)
43	format(/,' Lower level number: ',$)
21	format(A1)
c	************************************************************
c	Read all files and input parameters
c	************************************************************
	call predefine
100	kerr=0
	deltar=1.0
c	Here, we use only the z velocity component - to be improved
	call readfield(dco,vx,vy,vz,nx,ny,nz,kerr)
	if (kerr.ne.0) stop
	write(6,31)
	call extremes3d(dco,nx,ny,nz,values,point)
	write(6,32) values(1),values(2),values(3),values(4)
	call extremes3d(vx,nx,ny,nz,values,point)
	write(6,33) values(1),values(2),values(3),values(4)
	call extremes3d(vy,nx,ny,nz,values,point)
	write(6,33) values(1),values(2),values(3),values(4)
	call extremes3d(vz,nx,ny,nz,values,point)
	write(6,33) values(1),values(2),values(3),values(4)
c	Parameters for RT treatment
	mol=' '
	call readspecies(mol,lin,lev,kerr)
	if (kerr.ne.0) stop
	write(6,25) lev,lin
	call readphys(lup,llo,dvv,fwhm,dist,deltar,ztot,kerr)
	if (kerr.ne.0) stop 
c       Check transition
        ilin=theline(lup,llo,lin,msg)
        if (ilin.eq.0) goto 100
	call readnum(inum,kerr)
	if (kerr.ne.0) stop
c	************************************************************
c	Initialize fields
c	************************************************************
	dny=dvv
	call transobs(dny,fwhm,dist,deltar,sigbeam,off,kerr)
	if (kerr.ne.0) then
	 write(6,26)
	 goto 100
	endif
c	Translate into internal units, show extrema again
	call transphys(dco,vx,vy,vz,vt,t,dh,nx,ny,nz)
	write(6,31)
	call extremes3d(dco,nx,ny,nz,values,point)
	write(6,32) values(1),values(2),values(3),values(4)
	call extremes3d(vx,nx,ny,nz,values,point)
	write(6,33) values(1),values(2),values(3),values(4)
	call extremes3d(vy,nx,ny,nz,values,point)
	write(6,33) values(1),values(2),values(3),values(4)
	call extremes3d(vz,nx,ny,nz,values,point)
	write(6,33) values(1),values(2),values(3),values(4)
c	Initialize radiative units
	call uback(ubg,tback,lin)
	call initfreq(vz,vt,nx,ny,nz,dny,nff,dnyred,nffred,kerr)
	write(6,16) nffred,nff
	if (kerr.ne.0) then
	 write(6,12)
	 goto 100
	endif
c	***********************************************************
c	Initialize the tables for all densities
c	***********************************************************
	call maketables(dco,vx,vy,vz,vt,inx,nx,ny,nz,deltar,dnyred,
     %                     denstab,gradmom,regtab,freqmom,ntt,inum)
c	call printgradients(116,78,inx,denstab,gradmom,regtab)
	call initbeta
	do 120 l=1,lin
	do 120 i=1,ntt
120	uexi(i,l)=ubg(l)
	olev=lev
     	call ddtable(dh,t,uexi,cptb,sctb,lin,lev,
     %                          denstab,gradmom,regtab,ntt,inum)
c	No level reduction in the first guess
	lev=olev
c	***********************************************************
c	Iteration to compute the excitation 
c	Initialize kappa and S - two steps
c	***********************************************************
	if (inum.lt.0) goto 240
	iter=0
	i=0
150	continue
	i=i+1
	write(6,14)
	write(6,23) i,iter
c	************************************************************
c	Radiative transfer in the intensity iteration
c	************************************************************
	call stplev(dco,vz,vt,inx,cptb,sctb,uout,ubg,
     %		deltar,dnyred,ztot,nx,ny,nz,lin,nffred)
	write(6,24)
	call radexi(uout,uexi,freqmom,ntt,nffred,lin)
c	call printexi(102,83,inx,denstab,freqmom,uexi)
	call liter(uexi,ntt,lin,lev,iter)
c	call testu(uexi,ubg,ntt,lin)
	olev=lev
     	call ddtable(dh,t,uexi,cptb,sctb,lin,lev,
     %                          denstab,gradmom,regtab,ntt,inum)
c       reorder transition tables in case of level number reduction
	if (lev.ne.olev) then
	 if (iter.eq.1) then
	  call linfromlev(lintab,lin,lev)
	  call reducetrans(lintab,lin,ubg,cptb,sctb,ntt)
	  write(6,27) olev-lev
	  iter=0
	 else
	  lev=olev
	 endif
	endif
	if (iter.ge.0) goto 150
c	************************************************************
c	Ray tracing and beam integration
c	************************************************************
240	write(6,19)
c       Check transition
350	ilin=theline(lup,llo,lin,msg)
        if (ilin.eq.0) stop
        do 250 l=1,ntt
	cptabl(l)=cptb(l,ilin)
	sctabl(l)=sctb(l,ilin)
250	continue
	ubg1=ubg(ilin)
	call stpline(dco,vz,vt,inx,cptabl,sctabl,iout,ubg1,
     %		deltar,dny,ztot,nx,ny,nz,nff)
c	Statistical analysis of the intensity field
	write(6,31)
	call extremes3d(iout,nx,ny,-nff,values,point)
	write(6,34) values(1),values(2),values(3),values(4)
	vel=dny*(point(3)-nf-1)
c	vel=0.0
	call tauchek(dco,vz,vt,inx,cptabl,tau,vel,deltar,ztot,nx,ny,nz)
	call extremes2d(tau,nx,ny,values,point)
	write(6,35) values(1),values(2),values(3),values(4)
c	Beam integration
	write(6,15)
	factor=tbeam/cfreq(ilin)**2
	call intbeam(iout,ubg1,factor,sigbeam,nx,ny,nff)
	write(6,18) nx,ny
	call writeintens(iout,nx,ny,nff,off,dvv,msg)
	write(6,41)
	read(5,21) yn
	if ((yn.ne.'y').and.(yn.ne.'Y')) goto 300
	write(6,42)
	read(5,*) lup
	write(6,43)
	read(5,*)llo
	goto 350
300	write(6,17)
	read(5,21) yn
	if ((yn.eq.'y').or.(yn.eq.'Y')) goto 100
	stop
	end

C	***********************************************************
C	Line profile phi(ny) * ny0
c	! in the observers frame !
c	***********************************************************
c	For Falgarone intermittency profiles add:
c	Falgarone parameters f1=1+a, f2=a/3.3, f3=1/3.3^2
c	data f1,f2,f3 /1.3, 0.090909, 0.091827/
c	phi=pis/(sig*f1)*(exp(-nys)+f2*exp(-nys*f3))
c	***********************************************************
	real function phi(ny,vs,sig)
	implicit none
	real ny,nys,vs,sig
	real pis

c	1/sqrt(pi)
	data pis /0.5641896/

c	The cut-off for large negative exponents has to be treated
c	in the calling routine
	nys=((ny-vs)/sig)**2
	phi=pis/sig*exp(-nys)
	return
	end

c	***********************************************************
c	Small subroutine to print the radiation energy temperatures
c	***********************************************************
	subroutine testu(uexi,ubg,ntt,lev)
	implicit none
	integer lev,ntt

	include 'fsizes.inc'
	real uexi(ntab,ntra),ubg(ntra)
	real tbeam, f1,f2,f4,f7,cfreq

	data tbeam / 1.716215e9/

	f1=tbeam/cfreq(1)**2
	f2=tbeam/cfreq(2)**2
	f4=tbeam/cfreq(4)**2
	f7=tbeam/cfreq(7)**2
	write(6,*) 'Ubg:      ',f1*ubg(1),f2*ubg(2),f4*ubg(4),
     % f7*ubg(7)
	write(6,*) 'Urad_max: ',f1*uexi(1,1),f2*uexi(1,2),f4*uexi(1,4),
     % f7*uexi(1,7)
	write(6,*) 'Urad_min: ',f1*uexi(ntt,1),f2*uexi(ntt,2),
     % f4*uexi(ntt,4),f7*uexi(ntt,7)
	write(6,*)
	return
	end

