c	***********************************************************
c	Subroutine to carry out radiative transport computation
c	for one transition
c	This is a reduced version of stpmic for only one line
c	***********************************************************
	subroutine stpline(dco,vz,dlb,inx,cptb,sctb,iout,uext,
     %			deltar,dny,dztot,nx,ny,nz,nff)
	implicit none
	integer nff,nx,ny,nz

	include 'fsizes.inc'
	real vz(ns,ns,ns),dlb(ns,ns,ns),dco(ns,ns,ns)
	real cptb(ntab),sctb(ntab)
	real deltar,uext,dny,dztot
	real iout(ns,ns,-nf:nf)
	integer inx(ns,ns,ns)

	real cap(-nf:nf),iplus(-nf:nf)
	real cpo,sco,cp,sca

	real vs,dv,vso,dvo,dy2
	integer i,j,l,j1,j2,it,nztot
	integer kk,itstp,kstp

c	********************************************************
c	Get the number of total steps through the cloud
c	********************************************************
	nztot=int(dztot/deltar) 

c	********************************************************
c	This is the beginning of the loop over the whole field 
c	********************************************************
	do 400 i=1,nx
	do 410 j=1,ny

c	********************************************************
c	Beginning of the z-loop (integration along a ray)
c	********************************************************
c	Special treatment for the most outer left point
c	********************************************************
c	Geometry parameters
	kk=1
	vso = vz(i,j,kk)
	dvo = dlb(i,j,kk)
c	Background for all frequencies
	do 500 l=-nff,nff
500	iplus(l) = uext
c	Initialize kappa and source function fields
	cpo=dco(i,j,kk)*cptb(inx(i,j,kk))
	sco=sctb(inx(i,j,kk))
	call leftkp(cpo,cap,j1,j2,vso,dvo,dny)

c	********************************************************
c	Now integration - treat all points through the cloud
c	********************************************************
	do  1600  kstp = 2,nztot
	kk=mod(kstp-1,nz)+1
c	Initialize geometry parameters
	vs= vz(i,j,kk)
	dv = dlb(i,j,kk)
	cp=dco(i,j,kk)*cptb(inx(i,j,kk))
	sca=sctb(inx(i,j,kk))
c	*******************************************************
c	Radiative transfer - Multiple steps if the velocity
c	gradient is too large
c	*******************************************************
	call getstps(vso,dvo,cpo,sco,vs,dv,cp,sca,deltar,dy2,itstp)
c	Jumps
	if (itstp.eq.0) then
	call transf(cpo,sca,iplus,cap,sco,j1,j2,vso,dvo,dny,dy2)
	call nextstp(vso,dvo,cpo,sca)
	sco=sca
	call leftkp(cpo,cap,j1,j2,vso,dvo,dny)
	call transf(cpo,sca,iplus,cap,sco,j1,j2,vso,dvo,dny,dy2)
	else
	do 640 it=1,itstp
	call nextstp(vso,dvo,cpo,sca)
	call transf(cpo,sca,iplus,cap,sco,j1,j2,vso,dvo,dny,dy2)
640	continue
	endif
1600	continue
c	**********************************************************
c	End of the z loop
c	Store the results into a large field
c	**********************************************************
	do 420 l=-nff,nff
	iout(i,j,l) = iplus(l)
420	continue
410	continue
400	continue
	return
	end

c	************************************************************
c	Subroutine to compute the physical parameters at the end of 
c	the first radiative transfer step within one interval.
c	It checks how many steps have to be done there.
c	This is a reduced version of firststep
c	************************************************************
	subroutine getstps(vso,dvo,cpo,sco,vs,dv,cp,sc,dr,z,istp)
	implicit none
	integer istp

	real cpo,sco,cp,sc
	real vs,vso,dv,dvo,dr,z
	real deltav,vjump,explin,softzone
	real ddcp,ddsc,ddv,ddt

c	Internal variables are transmitted to nextstp for all
c	later steps
	common /stepl/ ddcp,ddsc,ddv,ddt
	common /accstep/ deltav,vjump,explin,softzone
	save /stepl/,/accstep/


c	*******************************************************
c	Jump or linear relation
c	*******************************************************
	ddv=vs-vso
	if (abs(ddv).gt.vjump*dv) then
	istp=0
	ddt=(dv-dvo)
	ddcp=(cp-cpo)
	ddsc=(sc-sco)
	z=0.25*dr
c	*******************************************************
c	linear approx 
c	*******************************************************
	else
	istp=int(abs(ddv)/(deltav*dv))+1
	ddv=ddv/istp
	ddt=(dv-dvo)/istp
	ddcp=(cp-cpo)/istp
	ddsc=(sc-sco)/istp
	z=0.5*dr/istp
	endif
	sc=sco
	return
	end

c	*********************************************************
c	Subroutine to compute the physical parameters at the
c	end of the following radiative transfer steps
c	*********************************************************
	subroutine nextstp(vso,dvo,cpo,sco)
	implicit none
	real cpo,sco
	real vso,dvo

c	Internal variables computed in firststep 
	real ddcp,ddsc,ddv,ddt
	common /stepl/ ddcp,ddsc,ddv,ddt
	save /stepl/

c	Update the physical parameters with firststep variables
	cpo=cpo+ddcp
	sco=sco+ddsc
	vso=vso+ddv
	dvo=dvo+ddt
	return
	end

c	***********************************************************
c	Initialization of the kappa field on the left cloud edge
c	This is a reduced version of leftkp for only one line
c	***********************************************************
	subroutine leftkp(cp,cappa,j1,j2,vs,dv,dny)
	implicit none
	integer j1,j2

	include 'fsizes.inc'
	real cp, cappa(-nf:nf)
	real vs,dv,dny
	real negexp,negint
	real phi,prof
	integer j

	common /accgauss/ negexp,negint
	save /accgauss/

c	Frequency range
	j1=int((vs-dv*negexp)/dny)-1
	j2=int((vs+dv*negexp)/dny)+1
c	Frequency dependent quantities
	do 500 j=j1,j2
	prof = phi(j*dny,vs,dv)
	cappa(j)  = cp * prof
500	continue
	return
	end

c	***********************************************************
c	Subroutine for radiative transfer between two points
c	This is a reduced version of transfer for only one line
c	***********************************************************
	subroutine transf(cp,sc,iplus,cap,ss,j1,j2,vs,dv,dny,z)
	implicit none
	integer j1,j2

	include 'fsizes.inc'
	real cp,sc,ss
	real cap(-nf:nf)
	real iplus(-nf:nf)
	real vs,dv,dny,z
	real negexp,negint,deltav,vjump,explin,softzone
	real phi,prof,tau1,tau2,tau,tauq
	real g1,ex1,ex2
	real ihlp1,ihlp2,rel
	integer jn1,jn2,jj1,jj2,j

	common /accstep/ deltav,vjump,explin,softzone
	common /accgauss/ negexp,negint
	save /accstep/,/accgauss/

c	*******************************************************
c	Determine indices of the frequency range
c	*******************************************************
	jn1=int((vs-dv*negexp)/dny)-1
	jn2=int((vs+dv*negexp)/dny)+1
c	Find the overlapping interval
	jj1=max(j1,jn1)
	jj2=min(j2,jn2)
c	*******************************************************
c	Treat the non-overlapping left end
c	*******************************************************
	if (jn1.lt.jj1) then
	do 510 j=jn1,jj1-1
	prof = phi(j*dny,vs,dv)
	cap(j) = cp*prof 
	tau = z*cap(j)
	iplus(j)=(iplus(j)+sc*tau)/(1.0+tau)
510	continue
	else if (j1.lt.jj1) then
	do 520 j=j1,jj1-1
	tau = z*cap(j)
	iplus(j)=(iplus(j)+ss*tau)/(1.0+tau)
520	continue
	endif
c	******************************************************
c	Treat the main, fully overlapping part
c	Only in this part, it is controlled whether the lines 
c	are optically thick, i.e. it is assumed that they are
c	thin within the non-overlapping wings
c	******************************************************
c	When even the centre of the line is optically 
c	thin the time consuming if in the inner loop shall
c	be avoided
c	******************************************************
	if (abs(2.0*z*cp*phi(0.0,0.0,dv)).lt.explin) then
	do 500 j=jj1,jj2
	prof = phi(j*dny,vs,dv)
	tau1 = z*cap(j)
	cap(j) = cp*prof 
	tau2 = z*cap(j)
	tauq = tau1*tau2
	iplus(j) = (iplus(j)+sc*tau2+ss*tau1
     %   +0.66667*tauq*(2.0*sc+ss))/(1.0+tau1+tau2+2.0*tauq)
500	continue
	else
c	******************************************************
c	In each step there is independently asked whether
c	the line is optically thick
c	It has to be guaranteed that none of the cappa values
c	is 0 in this case. (Should be always fulfilled for
c	the overlapping part.)
c	******************************************************
	do 1500 j=jj1,jj2
	prof = phi(j*dny,vs,dv)
	tau1 = z*cap(j)
	cap(j) = cp*prof 
	tau2 = z*cap(j)
	tau=tau1+tau2
	rel = abs(tau)
	if (rel.lt.explin) then
c	Thin case - linear approx.
	tauq = tau1*tau2
	iplus(j) = (iplus(j)+sc*tau2+ss*tau1
     %   +0.66667*tauq*(2.0*sc+ss))/(1.0+tau+2.0*tauq)
	else if (rel.lt.softzone) then
	tauq=tau1*tau2
c	Small transition regime - superposition
	ihlp1 = (iplus(j)+sc*tau2+ss*tau1
     %	 +0.66667*tauq*(2.0*sc+ss))/(1.0+tau+2.0*tauq)
	ex1=exp(-0.33333*tau1)
	ex2=exp(-0.33333*tau2)
	g1=0.5*(ss-sc)
	ihlp2=sc+ex2*ex2*(g1+ex2*ex1*(g1+ex1*ex1*(iplus(j)-ss)))
	iplus(j)=(ihlp1*(softzone-rel)+ihlp2*(rel-explin))/
     %              (softzone-explin)
	else
c	Optically thick case - exponential regime
	ex1=exp(-0.33333*tau1)
	ex2=exp(-0.33333*tau2)
	g1=0.5*(ss-sc)
	iplus(j)=sc+ex2*ex2*(g1+ex2*ex1*(g1+ex1*ex1*(iplus(j)-ss)))
	endif
1500	continue
	endif
c	*******************************************************
c	Treat the non-overlapping right end
c	*******************************************************
	if (jn2.gt.jj2) then
	do 560 j=jj2+1,jn2
	prof = phi(j*dny,vs,dv)
	cap(j) = cp*prof
	tau = z*cap(j) 
	iplus(j)=(iplus(j)+sc*tau)/(1.0+tau)
560	continue
	else if (j2.gt.jj2) then
	do 570 j=jj2+1,j2
	tau = z*cap(j)
	iplus(j)=(iplus(j)+ss*tau)/(1.0+tau)
570	continue
	endif
c	*******************************************************
c	New parameters
c	*******************************************************
	j1=jn1
	j2=jn2
	ss=sc
	return
	end
