c	***********************************************************
c	Subroutine to compute the line central optical depth
c	This is a reduced version of stpline 
c	***********************************************************
	subroutine tauchek(dco,vz,dlb,inx,cptb,taucent,
     %                             vel,deltar,dztot,nx,ny,nz)
	implicit none
	integer nx,ny,nz

	include 'fsizes.inc'
	real vz(ns,ns,ns),dlb(ns,ns,ns),dco(ns,ns,ns),taucent(ns,ns)
	real cptb(ntab)
	real deltar,dztot
	integer inx(ns,ns,ns)

	real cpo,cp,phi
	real vel,vs,dv,vso,dvo,dy2
	integer i,j,it,nztot
	integer kk,itstp,kstp

c	********************************************************
c	Get the number of total steps through the cloud
c	********************************************************
	nztot=int(dztot/deltar) 

c	********************************************************
c	This is the beginning of the loop over the whole field 
c	********************************************************
	do 400 i=1,nx
	do 410 j=1,ny
	taucent(i,j)=0.0

c	********************************************************
c	Beginning of the z-loop (integration along a ray)
c	********************************************************
c	Special treatment for the most outer left point
c	********************************************************
c	Geometry parameters
	kk=1
	vso = vz(i,j,kk)
	dvo = dlb(i,j,kk)
c	Initialize kappa and source function fields
	cpo=dco(i,j,kk)*cptb(inx(i,j,kk))

c	********************************************************
c	Now integration - treat all points through the cloud
c	********************************************************
	do  1600  kstp = 2,nztot
	kk=mod(kstp-1,nz)+1
c	Initialize geometry parameters
	vs = vz(i,j,kk)
	dv = dlb(i,j,kk)
	cp = dco(i,j,kk)*cptb(inx(i,j,kk))
c	*******************************************************
c	Radiative transfer - Multiple steps if the velocity
c	gradient is too large
c	*******************************************************
	call getkstps(vso,dvo,cpo,vs,dv,cp,deltar,dy2,itstp)
c	Jumps
	if (itstp.eq.0) then
	taucent(i,j)=taucent(i,j)+cpo*dy2*phi(vel,vso,dvo)
	call nextkstp(vso,dvo,cpo)
	taucent(i,j)=taucent(i,j)+cpo*dy2*phi(vel,vso,dvo)
	else
	do 640 it=1,itstp
	call nextkstp(vso,dvo,cpo)
	taucent(i,j)=taucent(i,j)+cpo*dy2*phi(vel,vso,dvo)
640	continue
	endif
1600	continue
c	**********************************************************
c	End of the z loop
c	**********************************************************
410	continue
400	continue
	return
	end

c	************************************************************
c	Subroutine to compute the physical parameters at the end of 
c	the first radiative transfer step within one interval.
c	It checks how many steps have to be done there.
c	This is a reduced version of firststep
c	************************************************************
	subroutine getkstps(vso,dvo,cpo,vs,dv,cp,dr,z,istp)
	implicit none
	integer istp

	real cpo,cp
	real vs,vso,dv,dvo,dr,z
	real deltav,vjump,explin,softzone
	real ddcp,ddv,ddt

c	Internal variables are transmitted to nextkstp for all
c	later steps
	common /kstepl/ ddcp,ddv,ddt
	common /accstep/ deltav,vjump,explin,softzone
	save /kstepl/,/accstep/

c	*******************************************************
c	Jump or linear relation
c	*******************************************************
	ddv=vs-vso
	if (abs(ddv).gt.vjump*dv) then
	istp=0
	ddt=(dv-dvo)
	ddcp=(cp-cpo)
	z=0.25*dr
c	*******************************************************
c	linear approx 
c	*******************************************************
	else
	istp=int(abs(ddv)/(deltav*dv))+1
	ddv=ddv/istp
	ddt=(dv-dvo)/istp
	ddcp=(cp-cpo)/istp
	z=0.5*dr/istp
	endif
	return
	end

c	*********************************************************
c	Subroutine to compute the physical parameters at the
c	end of the following radiative transfer steps
c	*********************************************************
	subroutine nextkstp(vso,dvo,cpo)
	implicit none
	real cpo
	real vso,dvo

c	Internal variables computed in firststep 
	real ddcp,ddv,ddt
	common /kstepl/ ddcp,ddv,ddt
	save /kstepl/

c	Update the physical parameters with firststep variables
	cpo=cpo+ddcp
	vso=vso+ddv
	dvo=dvo+ddt
	return
	end

